﻿using System;
using System.Windows;
using System.Windows.Interactivity;
using System.Windows.Media;
using Od.Behaviors.Utilities;

namespace Od.Behaviors.Animation
{
    /// <summary>
    /// Scale Animation
    /// Debug + Cleanup + dp props : OD 2010
    /// </summary>
    public class ScaleAnimationAction : TargetedTriggerAction<FrameworkElement>
    {
        
        #region ScaleSize Property 

        /// <summary>
        /// Gets or sets the scale size.
        /// </summary>
        /// <value>The size of the scale.</value>
        public double ScaleSize
        {
            get { return (double) GetValue(ScaleSizeProperty); }
            set { SetValue(ScaleSizeProperty, value); }
        }

        /// <summary>
        /// Gets or sets the scale size.
        /// </summary>
        public static readonly DependencyProperty ScaleSizeProperty =
            DependencyProperty.Register(
                "ScaleSize", typeof (double), typeof (ScaleAnimationAction),
                new PropertyMetadata(2d, onScaleSizeChanged));

        private static void onScaleSizeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {}

        #endregion


        #region Time Property 

        /// <summary>
        /// Gets or sets the animation duration.
        /// </summary>
        /// <value>The time.</value>
        public TimeSpan Time
        {
            get { return (TimeSpan) GetValue(TimeProperty); }
            set { SetValue(TimeProperty, value); }
        }

        /// <summary>
        ///  Gets or sets the animation duration.
        /// </summary>
        public static readonly DependencyProperty TimeProperty =
            DependencyProperty.Register(
                "Time", typeof (TimeSpan), typeof (ScaleAnimationAction),
                new PropertyMetadata(new TimeSpan(0,0,1), onTimeChanged));

        private static void onTimeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {}

        #endregion



        /// <summary>
        /// Invokes the action.
        /// </summary>
        /// <param name="parameter">The parameter to the action. If the action does not require a parameter, the parameter may be set to a <c>null</c> reference.</param>
        protected override void Invoke(object parameter)
        {
            // get the element
            var myElement = AssociatedObject as FrameworkElement;
            if (myElement == null) return;
            // create a new ScaleTransform
            myElement.RenderTransformOrigin = new Point(0.5, 0.5);
            myElement.RenderTransform = new ScaleTransform();

            // play the animation
            AnimationMaker.PlayAnimation(myElement, myElement.Name, "(UIElement.RenderTransform).(ScaleTransform.ScaleX)", Time, ScaleSize);
            AnimationMaker.PlayAnimation(myElement, myElement.Name, "(UIElement.RenderTransform).(ScaleTransform.ScaleY)", Time, ScaleSize);
        }
    }
}
