﻿using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interactivity;

namespace Od.Behaviors.Appearance
{
    /// <summary>
    /// Bring to front
    /// Debug + cleanup : OD 2010
    /// </summary>
    public class BringToFrontBehavior : Behavior<Panel>
    {
        /// <summary>
        /// Called after the behavior is attached to an AssociatedObject.
        /// </summary>
        /// <remarks>Override this to hook up functionality to the AssociatedObject.</remarks>
        protected override void OnAttached()
        {
            base.OnAttached();
            AssociatedObject.Loaded += associatedObject_Loaded;
        }

        /// <summary>
        /// Handles the Loaded event of the AssociatedObject control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        void associatedObject_Loaded(object sender, RoutedEventArgs e)
        {
            foreach (var child in AssociatedObject.Children)
            {
                child.MouseLeftButtonDown += child_MouseLeftButtonDown;
            }
        }

        static void child_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            bringToFront(sender);
        }

        /// <summary>
        /// Brings to front.
        /// </summary>
        /// <param name="sender">The sender.</param>
        private static void bringToFront(object sender)
        {
            var parent = (Panel)((FrameworkElement)sender).Parent;

            var z = getHighestValue(parent);

            foreach (var t in parent.Children.Where(t => sender.Equals(t)))
            {
                t.SetValue(Canvas.ZIndexProperty, z);
            }
        }

        private static int getHighestValue(Panel parent)
        {
            var highest = 0;
            foreach (var child in parent.Children)
            {
                var val = (int)child.GetValue(Canvas.ZIndexProperty);
                if (val > highest)
                {
                    highest = val;
                }
                child.SetValue(Canvas.ZIndexProperty, val - 1);
            }
            return highest;
        }
    }
}
