﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Interactivity;

namespace Od.Behaviors.Appearance
{
    /// <summary>
    /// Center and Scale
    /// Debug + cleanup + class name typo + detaching : OD 2010
    /// </summary>
    public class CenterAndScaleBehavior : Behavior<UserControl>
    {
        private UIElement oldContent;
        private Viewbox viewBox;
        private Grid grid;


        /// <summary>
        /// Called after the behavior is attached to an AssociatedObject.
        /// </summary>
        /// <remarks>Override this to hook up functionality to the AssociatedObject.</remarks>
        protected override void OnAttached()
        {
            oldContent = AssociatedObject.Content;
            base.OnAttached();
            AssociatedObject.Loaded += (s, e) =>
            {
                var uIElement = AssociatedObject.Content;
                AssociatedObject.Content = null;

                viewBox = new Viewbox
                                  {
                    Child = uIElement
                };

                grid = new Grid
                               {
                    HorizontalAlignment = HorizontalAlignment.Center,
                    VerticalAlignment = VerticalAlignment.Center
                };
                grid.Children.Add(viewBox);

                AssociatedObject.Content = grid;
            };
        }

        /// <summary>
        /// Called when the behavior is being detached from its AssociatedObject, but before it has actually occurred.
        /// </summary>
        /// <remarks>Override this to unhook functionality from the AssociatedObject.</remarks>
        protected override void OnDetaching()
        {
            if (viewBox != null) viewBox.Child = null;
            if (grid!=null) grid.Children.Clear();
            viewBox = null;
            grid = null;
            AssociatedObject.Content = oldContent;
            oldContent = null;
            base.OnDetaching();
        }
    }
}
