﻿using System;
using System.Windows;
using System.Windows.Interactivity;
using Od.Behaviors.Utilities;


namespace Od.Behaviors.Appearance
{
    /// <summary>
    /// Fade in/out animation
    /// Debug + cleanup + dp props : OD 2010
    /// </summary>
    public class FadeInOutAnimationAction : TargetedTriggerAction<FrameworkElement>
    {
        #region Time Property 

        /// <summary>
        /// Gets or sets the animation duration.
        /// </summary>
        /// <value>The time.</value>
        public TimeSpan Time
        {
            get { return (TimeSpan) GetValue(TimeProperty); }
            set { SetValue(TimeProperty, value); }
        }

        /// <summary>
        /// Gets or sets the animation duration.
        /// </summary>
        public static readonly DependencyProperty TimeProperty =
            DependencyProperty.Register(
                "Time", typeof (TimeSpan), typeof (FadeInOutAnimationAction),
                new PropertyMetadata(new TimeSpan(0,0,0,0,500), onTimeChanged));

        private static void onTimeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        { }

        #endregion

        #region FadeOption Property 

        /// <summary>
        /// Gets or sets the fade option.
        /// </summary>
        /// <value>The fade option.</value>
        public Fade FadeOption
        {
            get { return (Fade) GetValue(FadeOptionProperty); }
            set { SetValue(FadeOptionProperty, value); }
        }

        /// <summary>
        /// Gets or sets the fade option.
        /// </summary>
        public static readonly DependencyProperty FadeOptionProperty =
            DependencyProperty.Register(
                "FadeOption", typeof (Fade), typeof (FadeInOutAnimationAction),
                new PropertyMetadata(Fade.FadeOut, onFadeOptionChanged));

        private static void onFadeOptionChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        { }

        #endregion

        #region Enum
        public enum Fade { FadeIn, FadeOut }; 
        #endregion

        /// <summary>
        /// Invokes the action.
        /// </summary>
        /// <param name="parameter">The parameter to the action. If the action does not require a parameter, the parameter may be set to a <c>null</c> reference.</param>
        protected override void Invoke(object parameter)
        {
            // get the element that must be animated
            var myElement = AssociatedObject as FrameworkElement;

            // check if we need to fade in or fade out
            var value = FadeOption == Fade.FadeIn ? 1d : 0d;

            // start the animation
            if (myElement != null) AnimationMaker.PlayAnimation(myElement, myElement.Name, "Opacity", Time, value);
        }
    }
}
