﻿using System;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Shapes;

namespace Od.Behaviors.Appearance
{
    //todo : Review all texts, not good english.
    /// <summary>
    /// Resize behavior
    /// Debug + cleanup : OD 2010 
    /// </summary>
    public class ResizeBehavior : Behavior<FrameworkElement>
    {
        private enum ControlPart
        {
            None,
            Left,
            Right,
            Top,
            Bottom,
            LeftTop,
            LeftBottom,
            RightTop,
            RightBottom,
        }

        bool isResizing;
        ControlPart resizePart;

        #region  Properties

        #region Size Properties

        #region MinHeight

        /// <summary>
        /// Set the minimum resize height
        /// </summary>
        public static readonly DependencyProperty MinHeightProperty =
            DependencyProperty.Register("MinHeight", typeof(Double), typeof(ResizeBehavior),
            new PropertyMetadata(15.0, onSizeChanged));

        /// <summary>
        /// Set the minimum resize height
        /// </summary>
        /// <value>The height of the min.</value>
        [Category("Size"), Description("Set the minimum resize height")]
        public double MinHeight
        {
            get { return (double)GetValue(MinHeightProperty); }
            set { SetValue(MinHeightProperty, value); }
        }

        #endregion

        #region MaxHeight

        /// <summary>
        /// Set the maximum resize height
        /// </summary>
        public static readonly DependencyProperty MaxHeightProperty =
            DependencyProperty.Register("MaxHeight", typeof(Double), typeof(ResizeBehavior),
            new PropertyMetadata(400.0, onSizeChanged));

        /// <summary>
        /// Set the maximum resize height
        /// </summary>
        /// <value>The height of the max.</value>
        [Category("Size"), Description("Set the maximum resize height")]
        public double MaxHeight
        {
            get { return (double)GetValue(MaxHeightProperty); }
            set { SetValue(MaxHeightProperty, value); }
        }

        #endregion

        #region MinWidth

        /// <summary>
        /// Set the minimum resize width
        /// </summary>
        public static readonly DependencyProperty MinWidthProperty =
            DependencyProperty.Register("MinWidth", typeof(Double), typeof(ResizeBehavior),
            new PropertyMetadata(15.0, onSizeChanged));

        /// <summary>
        /// Set the minimum resize width
        /// </summary>
        /// <value>The width of the min.</value>
        [Category("Size"), Description("Set the minimum resize width")]
        public double MinWidth
        {
            get { return (double)GetValue(MinWidthProperty); }
            set { SetValue(MinWidthProperty, value); }
        }

        #endregion

        #region MaxWidth

        /// <summary>
        /// Set the maximum resize width
        /// </summary>
        public static readonly DependencyProperty MaxWidthProperty =
            DependencyProperty.Register("MaxWidth", typeof(Double), typeof(ResizeBehavior),
            new PropertyMetadata(400.0, onSizeChanged));

        /// <summary>
        /// Set the maximum resize width
        /// </summary>
        /// <value>The width of the max.</value>
        [Category("Size"), Description("Set the maximum resize width")]
        public double MaxWidth
        {
            get { return (double)GetValue(MaxWidthProperty); }
            set { SetValue(MaxWidthProperty, value); }
        }

        #endregion

        private static void onSizeChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            var source = (ResizeBehavior)obj;
            source.checkSizeValues();
        }

        #endregion

        #region Appareance Properties

        #region DragSpace

        /// <summary>
        /// Set the space from border to start resize mode
        /// </summary>
        public static readonly DependencyProperty DragSpaceProperty =
            DependencyProperty.Register("DragSpace", typeof(double), typeof(ResizeBehavior),
            new PropertyMetadata(7.0, onAppareanceElementChanged));

        /// <summary>
        /// Set the space from border for start resize mode
        /// </summary>
        /// <value>The drag space.</value>
        [Category("Appearance"), Description("Set the space from border to start resize mode")]
        public double DragSpace
        {
            get { return (double)GetValue(DragSpaceProperty); }
            set { SetValue(DragSpaceProperty, value); }
        }



        #endregion

        #region DragIndicatorsFill

        /// <summary>
        /// Set the drags indicator backGround color
        /// </summary>
        public static readonly DependencyProperty DragIndicatorsFillProperty =
            DependencyProperty.Register("DragIndicatorsFill", typeof(Brush), typeof(ResizeBehavior),
            new PropertyMetadata(new SolidColorBrush(Colors.Yellow), onAppareanceElementChanged));

        /// <summary>
        /// Set the drags indicator backGround color
        /// </summary>
        /// <value>The drag indicators fill.</value>
        [Category("Appearance"), Description("Set the drags indicator backGround color")]
        public Brush DragIndicatorsFill
        {
            get { return (Brush)GetValue(DragIndicatorsFillProperty); }
            set { SetValue(DragIndicatorsFillProperty, value); }
        }

        #endregion

        #region DragIndicatorsOpacity

        /// <summary>
        /// Set the drags indicator opacity
        /// </summary>
        public static readonly DependencyProperty DragIndicatorsOpacityProperty =
            DependencyProperty.Register("DragIndicatorsOpacity", typeof(double), typeof(ResizeBehavior),
            new PropertyMetadata(0.8, onAppareanceElementChanged));

        /// <summary>
        /// Set the drags indicator opacity
        /// </summary>
        /// <value>The drag indicators opacity.</value>
        [Category("Appearance"), Description("Set the drags indicators opacity")]
        public Double DragIndicatorsOpacity
        {
            get { return (Double)GetValue(DragIndicatorsOpacityProperty); }
            set { SetValue(DragIndicatorsOpacityProperty, value); }
        }

        #endregion


        private static void onAppareanceElementChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            var source = (ResizeBehavior)obj;
            if (source.AssociatedObject == null) return;
            source.setRectangle(source.resizePart);
        }

        #endregion

        #region behavior

        #region IsLeftDraggable

        /// <summary>
        ///  Indicates whether left handle is accessible to resize the underlying object
        /// </summary>
        public static readonly DependencyProperty IsLeftDraggableProperty =
            DependencyProperty.Register("IsLeftDraggable", typeof(bool), typeof(ResizeBehavior),
            new PropertyMetadata(true));

        /// <summary>
        /// Indicates whether left handle is accessible to resize the underlying object
        /// </summary>
        /// <value>
        /// 	<c>true</c> if left handle is accessible; otherwise, <c>false</c>.
        /// </value>
        [Category("Behavior"), Description("Indicates whether left handle is accessible to resize the underlying object")]
        public bool IsLeftDraggable
        {
            get { return (bool)GetValue(IsLeftDraggableProperty); }
            set { SetValue(IsLeftDraggableProperty, value); }
        }

        #endregion

        #region IsRightDraggable

        /// <summary>
        /// Indicates whether right handle is accessible to resize the underlying object
        /// </summary>
        public static readonly DependencyProperty IsRightDraggableProperty =
            DependencyProperty.Register("IsRightDraggable", typeof(bool), typeof(ResizeBehavior),
            new PropertyMetadata(true));

        /// <summary>
        /// Indicates whether right handle is accessible to resize the underlying object
        /// </summary>
        /// <value>
        /// 	<c>true</c> if right handle is accessible; otherwise, <c>false</c>.
        /// </value>
        [Category("Behavior"), Description("Indicates whether right handle is accessible to resize the underlying object")]
        public bool IsRightDraggable
        {
            get { return (bool)GetValue(IsRightDraggableProperty); }
            set { SetValue(IsRightDraggableProperty, value); }
        }

        #endregion

        #region IsTopDraggable

        /// <summary>
        /// Indicates whether top handle is accessible to resize the underlying object
        /// </summary>
        public static readonly DependencyProperty IsTopDraggableProperty =
            DependencyProperty.Register("IsTopDraggable", typeof(bool), typeof(ResizeBehavior),
            new PropertyMetadata(true));

        /// <summary>
        /// Indicates whether top handle is accessible to resize the underlying object
        /// </summary>
        /// <value>
        /// 	<c>true</c> if top handle is accessible; otherwise, <c>false</c>.
        /// </value>
        [Category("Behavior"), Description("Indicates whether top handle is accessible to resize the underlying object")]
        public bool IsTopDraggable
        {
            get { return (bool)GetValue(IsTopDraggableProperty); }
            set { SetValue(IsTopDraggableProperty, value); }
        }

        #endregion

        #region IsBottomDraggable

        /// <summary>
        /// Indicates whether bottom handle is accessible to resize the underlying object
        /// </summary>
        public static readonly DependencyProperty IsBottomDraggableProperty =
            DependencyProperty.Register("IsBottomDraggable", typeof(bool), typeof(ResizeBehavior),
            new PropertyMetadata(true));

        /// <summary>
        /// Indicates whether bottom handle is accessible to resize the underlying object
        /// </summary>
        /// <value>
        /// 	<c>true</c> if bottom handle is accessible; otherwise, <c>false</c>.
        /// </value>
        [Category("Behavior"), Description("Indicates whether bottom handle is accessible to resize the underlying object")]
        public bool IsBottomDraggable
        {
            get { return (bool)GetValue(IsBottomDraggableProperty); }
            set { SetValue(IsBottomDraggableProperty, value); }
        }

        #endregion

        #region IsTopLeftDraggable

        /// <summary>
        /// Indicates whether top left handle is accessible to resize the underlying object
        /// </summary>
        public static readonly DependencyProperty IsTopLeftDraggableProperty =
            DependencyProperty.Register("IsTopLeftDraggable", typeof(bool), typeof(ResizeBehavior),
            new PropertyMetadata(true));

        /// <summary>
        /// Indicates whether top left handle is accessible to resize the underlying object
        /// </summary>
        /// <value>
        /// 	<c>true</c> if top left handle is accessible; otherwise, <c>false</c>.
        /// </value>
        [Category("Behavior"), Description("Indicates whether top left handle is accessible to resize the underlying object")]
        public bool IsTopLeftDraggable
        {
            get { return (bool)GetValue(IsTopLeftDraggableProperty); }
            set { SetValue(IsTopLeftDraggableProperty, value); }
        }

        #endregion

        #region IsTopRightDraggable

        /// <summary>
        ///  Indicates whether top right handle is accessible to resize the underlying object
        /// </summary>
        public static readonly DependencyProperty IsTopRightDraggableProperty =
            DependencyProperty.Register("IsTopRightDraggable", typeof(bool), typeof(ResizeBehavior),
            new PropertyMetadata(true));

        /// <summary>
        /// Indicates whether top right handle is accessible to resize the underlying object
        /// </summary>
        /// <value>
        /// 	<c>true</c> if top right handle is accessible; otherwise, <c>false</c>.
        /// </value>
        [Category("Behavior"), Description("Indicates whether top left handle is accessible to resize the underlying object")]
        public bool IsTopRightDraggable
        {
            get { return (bool)GetValue(IsTopRightDraggableProperty); }
            set { SetValue(IsTopRightDraggableProperty, value); }
        }

        #endregion

        #region IsBottomLeftDraggable

        /// <summary>
        /// Indicates whether bottom left handle is accessible to resize the underlying object
        /// </summary>
        public static readonly DependencyProperty IsBottomLeftDraggableProperty =
            DependencyProperty.Register("IsBottomLeftDraggable", typeof(bool), typeof(ResizeBehavior),
            new PropertyMetadata(true));

        /// <summary>
        /// Indicates whether bottom left handle is accessible to resize the underlying object
        /// </summary>
        [Category("Behavior"), Description("Indicates whether bottom left handle is accessible to resize the underlying object")]
        public bool IsBottomLeftDraggable
        {
            get { return (bool)GetValue(IsBottomLeftDraggableProperty); }
            set { SetValue(IsBottomLeftDraggableProperty, value); }
        }

        #endregion

        #region IsBottomRightDraggable

        /// <summary>
        /// Indicates whether bottom right handle is accessible to resize the underlying object
        /// </summary>
        public static readonly DependencyProperty IsBottomRightDraggableProperty =
            DependencyProperty.Register("IsBottomRightDraggable", typeof(bool), typeof(ResizeBehavior),
            new PropertyMetadata(true));

        /// <summary>
        /// Indicates whether bottom right handle is accessible to resize the underlying object
        /// </summary>
        [Category("Behavior"), Description("Indicates whether bottom right handle is accessible to resize the underlying object")]
        public bool IsBottomRightDraggable
        {
            get { return (bool)GetValue(IsBottomRightDraggableProperty); }
            set { SetValue(IsBottomRightDraggableProperty, value); }
        }

        #endregion


        #region StayInParent

        /// <summary>
        /// Indicates if the element stay or not into parent bounds
        /// </summary>
        public static readonly DependencyProperty StayInParentProperty =
            DependencyProperty.Register("StayInParent", typeof(bool), typeof(ResizeBehavior),
            new PropertyMetadata(true, stayInParentPropertyChanged));

        /// <summary>
        /// Indicates if the element stay or not into parent bounds
        /// </summary>
        [Category("Behavior"), Description("Indicate if the element stay or not into parent bounds")]
        public bool StayInParent
        {
            get { return (bool)GetValue(StayInParentProperty); }
            set { SetValue(StayInParentProperty, value); }
        }


        private static void stayInParentPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            var source = (ResizeBehavior)obj;
            source.checkParentBounds();
        }

        #endregion

        #endregion

        #endregion

        #region Events

        // 
        /// <summary>
        /// Exposes a SizeChanged event when size is changed by behavior the <see cref="FrameworkElement"/>.SizeChanged event
        /// is not raised.
        /// </summary>
        [Category("Size"), Description("Exposes a SizeChanged event when size is changed by behavior the FrameworkElement.SizeChanged event is not raised")]
        public event EventHandler SizeChanged;

        private void raiseEvent(EventHandler e)
        {
            raiseEvent(e, EventArgs.Empty);
        }

        private void raiseEvent(EventHandler e, EventArgs args)
        {
            if (e != null) e(this, args);
        }

        #endregion

        #region behaviour elements implementation

        /// <summary>
        /// Called after the behavior is attached to an AssociatedObject.
        /// </summary>
        /// <remarks>Override this to hook up functionality to the AssociatedObject.</remarks>
        protected override void OnAttached()
        {
            base.OnAttached();

            isResizing = false;
            resizePart = ControlPart.None;
            setRectangle(ControlPart.None);

            checkSizeValues();
            checkParentBounds();

            AssociatedObject.MouseLeftButtonDown += associatedObject_MouseLeftButtonDown;
            AssociatedObject.MouseLeftButtonUp += associatedObject_MouseLeftButtonUp;
            AssociatedObject.MouseMove += associatedObject_MouseMove;
            AssociatedObject.MouseLeave += associatedObject_MouseLeave;

            // bind to this event because the parent of the associated object can be retrieve only when the latter is loaded
            AssociatedObject.Loaded += associatedObject_Loaded;
        }

        void associatedObject_Loaded(object sender, RoutedEventArgs e)
        {
            AssociatedObject.Loaded -= associatedObject_Loaded;

            var parentAttachedBehaviors = Interaction.GetBehaviors(AssociatedObject.Parent);

            var parentResizeBehavior = parentAttachedBehaviors.Where(b => b.GetType() == typeof(ResizeBehavior)).Cast<ResizeBehavior>().FirstOrDefault();

            if (parentResizeBehavior == null)
                ((FrameworkElement)AssociatedObject.Parent).SizeChanged += parentSizeChanged;

            else
                //attach to parent's resize behavior to notify size changes
                parentResizeBehavior.SizeChanged += parentBehaviorSizeChanged;
        }


        /// <summary>
        /// Called when the behavior is being detached from its AssociatedObject, but before it has actually occurred.
        /// </summary>
        /// <remarks>Override this to unhook functionality from the AssociatedObject.</remarks>
        protected override void OnDetaching()
        {
            base.OnDetaching();

            AssociatedObject.MouseLeftButtonDown -= associatedObject_MouseLeftButtonDown;
            AssociatedObject.MouseLeftButtonUp -= associatedObject_MouseLeftButtonUp;
            AssociatedObject.MouseMove -= associatedObject_MouseMove;
            AssociatedObject.MouseLeave -= associatedObject_MouseLeave;
        }

        #endregion


        #region Associated object events

        void associatedObject_MouseMove(object sender, MouseEventArgs e)
        {
            if (isResizing)
            {
                resize(e.GetPosition(AssociatedObject), e.GetPosition((FrameworkElement)AssociatedObject.Parent));
                setRectangle(resizePart);
            }
            else
            {
                setRectangle(getResizePart(e.GetPosition(AssociatedObject)));
                setCursor(getResizePart(e.GetPosition(AssociatedObject)));
            }
        }

        void associatedObject_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            AssociatedObject.ReleaseMouseCapture();
            isResizing = false;
            resizePart = ControlPart.None;
            setRectangle(ControlPart.None);
        }

        void associatedObject_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            AssociatedObject.CaptureMouse();
            isResizing = true;
            resizePart = getResizePart(e.GetPosition(AssociatedObject));

            e.Handled = true;
        }

        void associatedObject_MouseLeave(object sender, MouseEventArgs e)
        {
            setRectangle(ControlPart.None);
        }


        

        #endregion

        #region Associated object parent events

        void parentSizeChanged(object sender, SizeChangedEventArgs e)
        {
            checkParentBounds();
        }

        void parentBehaviorSizeChanged(object sender, EventArgs e)
        {
            checkParentBounds();
        }

        #endregion


        #region resizing functions

        ControlPart getResizePart(Point mousePosition)
        {
            var size = new Size(
                AssociatedObject.ActualWidth,
                AssociatedObject.ActualHeight);

            var center = new Point(size.Width / 2, size.Height / 2);

            if (mousePosition.X > center.X && mousePosition.X > size.Width - DragSpace &&
                mousePosition.Y < center.Y && mousePosition.Y < DragSpace &&
                IsTopRightDraggable)
                return ControlPart.RightTop;

            if (mousePosition.X > center.X && mousePosition.X > size.Width - DragSpace &&
                mousePosition.Y > center.Y && mousePosition.Y > size.Height - DragSpace &&
                IsBottomRightDraggable)
                return ControlPart.RightBottom;

            if (mousePosition.X < center.X && mousePosition.X < DragSpace &&
                mousePosition.Y < center.Y && mousePosition.Y < DragSpace &&
                IsTopLeftDraggable)
                return ControlPart.LeftTop;

            if (mousePosition.X < center.X && mousePosition.X < DragSpace &&
                mousePosition.Y > center.Y && mousePosition.Y > size.Height - DragSpace &&
                IsBottomLeftDraggable)
                return ControlPart.LeftBottom;

            if (mousePosition.X > center.X && mousePosition.X > size.Width - DragSpace &&
                IsRightDraggable)
                return ControlPart.Right;

            if (mousePosition.X < center.X && mousePosition.X < DragSpace &&
                IsLeftDraggable)
                return ControlPart.Left;

            if (mousePosition.Y < center.Y && mousePosition.Y < DragSpace &&
                IsTopDraggable)
                return ControlPart.Top;

            if (mousePosition.Y > center.Y && mousePosition.Y > size.Height - DragSpace &&
                IsBottomDraggable)
                return ControlPart.Bottom;

            return ControlPart.None;
        }

        void setCursor(ControlPart part)
        {
            switch (part)
            {
                case ControlPart.None:
                    AssociatedObject.Cursor = Cursors.Arrow;
                    break;

                case ControlPart.Top:
                case ControlPart.Bottom:
                    AssociatedObject.Cursor = Cursors.SizeNS;
                    break;

                case ControlPart.LeftBottom:
                case ControlPart.LeftTop:
                case ControlPart.RightBottom:
                case ControlPart.RightTop:
                case ControlPart.Right:
                case ControlPart.Left:
                    AssociatedObject.Cursor = Cursors.SizeWE;
                    break;
            }

        }

        void setRectangle(ControlPart part)
        {
            var parent = AssociatedObject.Parent as Canvas;

            if (parent == null) return;

            for (var i = 0; i < parent.Children.Count; i++)
            {
                var e = parent.Children[i] as FrameworkElement;

                if (e == null) continue;

                if (e.Name != "__RESIZE_AUTO_GEN_RECTANGLE" + GetHashCode()) continue;
                e.Visibility = Visibility.Collapsed;
                parent.Children.RemoveAt(i);
                break;
            }

            var rect = new Rectangle
                           {
                               Name = "__RESIZE_AUTO_GEN_RECTANGLE" + GetHashCode(),
                               Fill = DragIndicatorsFill,
                               Opacity = DragIndicatorsOpacity
                           };

            rect.SetValue(Canvas.ZIndexProperty, (int)AssociatedObject.GetValue(Canvas.ZIndexProperty) + 1);
            rect.IsHitTestVisible = false;
            rect.RenderTransformOrigin = AssociatedObject.RenderTransformOrigin;
            rect.RenderTransform = AssociatedObject.RenderTransform;

            switch (part)
            {
                case ControlPart.None:
                    rect = null;
                    break;

                case ControlPart.Top:
                    rect.Width = AssociatedObject.ActualWidth;
                    rect.Height = DragSpace;
                    rect.SetValue(Canvas.TopProperty, AssociatedObject.GetValue(Canvas.TopProperty));
                    rect.SetValue(Canvas.LeftProperty, AssociatedObject.GetValue(Canvas.LeftProperty));
                    break;

                case ControlPart.Bottom:
                    rect.Width = AssociatedObject.ActualWidth;
                    rect.Height = DragSpace;
                    rect.SetValue(Canvas.TopProperty, AssociatedObject.ActualHeight - DragSpace + (double)AssociatedObject.GetValue(Canvas.TopProperty));
                    rect.SetValue(Canvas.LeftProperty, AssociatedObject.GetValue(Canvas.LeftProperty));
                    break;

                case ControlPart.Right:
                    rect.Width = DragSpace;
                    rect.Height = AssociatedObject.ActualHeight;
                    rect.SetValue(Canvas.TopProperty, AssociatedObject.GetValue(Canvas.TopProperty));
                    rect.SetValue(Canvas.LeftProperty, AssociatedObject.ActualWidth - DragSpace + (double)AssociatedObject.GetValue(Canvas.LeftProperty));
                    break;

                case ControlPart.Left:
                    rect.Width = DragSpace;
                    rect.Height = AssociatedObject.ActualHeight;
                    rect.SetValue(Canvas.TopProperty, AssociatedObject.GetValue(Canvas.TopProperty));
                    rect.SetValue(Canvas.LeftProperty, AssociatedObject.GetValue(Canvas.LeftProperty));
                    break;

                case ControlPart.RightTop:
                    rect.Width = DragSpace;
                    rect.Height = DragSpace;
                    rect.SetValue(Canvas.TopProperty, AssociatedObject.GetValue(Canvas.TopProperty));
                    rect.SetValue(Canvas.LeftProperty, AssociatedObject.ActualWidth - DragSpace + (double)AssociatedObject.GetValue(Canvas.LeftProperty));
                    break;

                case ControlPart.RightBottom:
                    rect.Width = DragSpace;
                    rect.Height = DragSpace;
                    rect.SetValue(Canvas.TopProperty, AssociatedObject.ActualHeight - DragSpace + (double)AssociatedObject.GetValue(Canvas.TopProperty));
                    rect.SetValue(Canvas.LeftProperty, AssociatedObject.ActualWidth - DragSpace + (double)AssociatedObject.GetValue(Canvas.LeftProperty));
                    break;

                case ControlPart.LeftBottom:
                    rect.Width = DragSpace;
                    rect.Height = DragSpace;
                    rect.SetValue(Canvas.TopProperty, AssociatedObject.ActualHeight - DragSpace + (double)AssociatedObject.GetValue(Canvas.TopProperty));
                    rect.SetValue(Canvas.LeftProperty, AssociatedObject.GetValue(Canvas.LeftProperty));
                    break;

                case ControlPart.LeftTop:
                    rect.Width = DragSpace;
                    rect.Height = DragSpace;
                    rect.SetValue(Canvas.TopProperty, AssociatedObject.GetValue(Canvas.TopProperty));
                    rect.SetValue(Canvas.LeftProperty, AssociatedObject.GetValue(Canvas.LeftProperty));
                    break;
            }

            if (rect == null) return;

            parent.Children.Add(rect);
        }

        void resize(Point insideMousePosition, Point parentMousePosition)
        {
            if (!(AssociatedObject.Parent is Canvas)) return;
            var newHeight = AssociatedObject.ActualHeight;
            var newWidth = AssociatedObject.ActualWidth;
            var newPosition = new Point(
                (double)AssociatedObject.GetValue(Canvas.LeftProperty),
                (double)AssociatedObject.GetValue(Canvas.TopProperty));



            switch (resizePart)
            {
                case ControlPart.RightTop:
                    newWidth = insideMousePosition.X;
                    newHeight -= parentMousePosition.Y - newPosition.Y;
                    newPosition.Y = parentMousePosition.Y;
                    break;

                case ControlPart.RightBottom:
                    newWidth = insideMousePosition.X;
                    newHeight = insideMousePosition.Y;
                    break;

                case ControlPart.LeftTop:
                    newWidth -= parentMousePosition.X - newPosition.X;
                    newPosition.X = parentMousePosition.X;
                    newHeight -= parentMousePosition.Y - newPosition.Y;
                    newPosition.Y = parentMousePosition.Y;
                    break;

                case ControlPart.LeftBottom:
                    newWidth -= parentMousePosition.X - newPosition.X;
                    newPosition.X = parentMousePosition.X;
                    newHeight = insideMousePosition.Y;
                    break;

                case ControlPart.Right:
                    newWidth = insideMousePosition.X;
                    break;

                case ControlPart.Left:
                    newWidth += newPosition.X - parentMousePosition.X;
                    newPosition.X = parentMousePosition.X;
                    break;

                case ControlPart.Top:
                    newHeight -= parentMousePosition.Y - newPosition.Y;
                    newPosition.Y = parentMousePosition.Y;
                    break;

                case ControlPart.Bottom:
                    newHeight = insideMousePosition.Y;
                    break;
            }

            var hasChanges = false;

            if (newWidth >= MinWidth && newWidth <= MaxWidth)
            {
                AssociatedObject.Width = newWidth;
                AssociatedObject.SetValue(Canvas.LeftProperty, newPosition.X);
                hasChanges = true;
            }

            if (newHeight >= MinHeight && newHeight <= MaxHeight)
            {
                AssociatedObject.Height = newHeight;
               AssociatedObject.SetValue(Canvas.TopProperty, newPosition.Y);
                hasChanges = true;
            }

            // call this to notify size change
            if (hasChanges)
                raiseEvent(SizeChanged);

            checkParentBounds();
        }

        #endregion


        #region check functions

        void checkParentBounds()
        {
            if (!StayInParent || AssociatedObject == null) return;

            var parent = AssociatedObject.Parent as Canvas;

            // check if parent exist and is a canvas
            if (parent == null) return;

            var hasChanges = false;

            try
            {
                if (Canvas.GetLeft(AssociatedObject) < 0)
                {
                    var prvLeft = Canvas.GetLeft(AssociatedObject);
                    Canvas.SetLeft(AssociatedObject, 0);
                    AssociatedObject.Width += prvLeft;
                    hasChanges = true;
                }

                if (Canvas.GetTop(AssociatedObject) < 0)
                {
                    var prvTop = Canvas.GetTop(AssociatedObject);
                    Canvas.SetTop(AssociatedObject, 0);
                    AssociatedObject.Height += prvTop;
                    hasChanges = true;
                }

                if (Canvas.GetLeft(AssociatedObject) + AssociatedObject.Width > parent.ActualWidth)
                {
                    AssociatedObject.Width = parent.ActualWidth - Canvas.GetLeft(AssociatedObject);
                    hasChanges = true;
                }


                if (Canvas.GetTop(AssociatedObject) + AssociatedObject.Height > parent.ActualHeight)
                {
                    AssociatedObject.Height = parent.ActualHeight - Canvas.GetTop(AssociatedObject);
                    hasChanges = true;
                }

            }
            catch (Exception)
            {
                hasChanges = false;
            }

            // call this to notify size change
            if (hasChanges)
                raiseEvent(SizeChanged);

        }

        void checkSizeValues()
        {
            var element = AssociatedObject;

            if (element == null) return;

            var hasChanges = false;

            if (element.Width > MaxWidth)
            {
                element.Width = MaxWidth;
                hasChanges = true;
            }

            if (element.Width < MinWidth)
            {
                element.Width = MinWidth;
                hasChanges = true;
            }

            if (element.Height > MaxHeight)
            {
                element.Height = MaxHeight;
                hasChanges = true;
            }

            if (element.Height < MinHeight)
            {
                element.Height = MinHeight;
                hasChanges = true;
            }

            // call this to notify size change
            if (hasChanges)
                raiseEvent(SizeChanged);
        }

        #endregion
    }
}
