﻿using System.ComponentModel;
using System.Windows.Interactivity;
using System.Windows;
using Od.Behaviors.Utilities;

namespace Od.Behaviors.Appearance
{
    /// <summary>
    /// Scale Action
    /// Cleanup : OD 2010
    /// </summary>
    public class ScaleAction : TargetedTriggerAction<UIElement>
    {
        /// <summary>
        /// Invokes the action.
        /// </summary>
        /// <param name="parameter">The parameter to the action. If the action does not require a parameter, the parameter may be set to a <see langword="null"/> reference.</param>
        protected override void Invoke(object parameter)
        {
            StoryBoardHelper.ScaleAnimation(Target, Percent, Duration);
        }

        /// <summary>
        /// Gets or sets the percent.
        /// </summary>
        /// <value>The percent.</value>
        [Description("Scaling percentage")]
        public double Percent
        {
            get { return (double)GetValue(PercentProperty); }
            set { SetValue(PercentProperty, value); }
        }

        /// <summary>
        ///  Gets or sets the percent.
        /// </summary>
        public static readonly DependencyProperty PercentProperty =
            DependencyProperty.Register("Percent", typeof(double), typeof(ScaleAction), new PropertyMetadata((double)1));


        #region duration Property 

        /// <summary>
        /// Gets or sets the duration.
        /// </summary>
        /// <value>The duration.</value>
        [Description("Animation duration in milliseconds")]
        public double Duration
        {
            get { return (double) GetValue(DurationProperty); }
            set { SetValue(DurationProperty, value); }
        }

        /// <summary>
        /// Gets or sets the duration.
        /// </summary>
        public static readonly DependencyProperty DurationProperty =
            DependencyProperty.Register(
                "Duration", typeof (double), typeof (ScaleAction),
                new PropertyMetadata(200d, null));

        
        #endregion


    }
}
