﻿using System.ComponentModel;
using System.Windows;
using System.Windows.Interactivity;

namespace Od.Behaviors.Appearance
{
    /// <summary>
    /// Toggle opacity
    /// Cleanup : OD 2010
    /// </summary>
    public class ToggleOpacityAction : TargetedTriggerAction<UIElement>
    {
        private double originalOpacity;

        /// <summary>
        /// Invokes the action.
        /// </summary>
        /// <param name="parameter">The parameter to the action. If the action does not require a parameter, the parameter may be set to a <c>null</c> reference.</param>
        protected override void Invoke(object parameter)
        {
            Target.Opacity = areDoubleEquals(originalOpacity, Target.Opacity, 1000) ? AlternateOpacity : originalOpacity;
        }

        private static bool areDoubleEquals(double a, double b, int factor)
        {
            var ia = (int) (a*factor);
            var ib = (int) (b*factor);
            return ia == ib;
        }

        /// <summary>
        /// Called after the action is attached to an AssociatedObject.
        /// </summary>
        protected override void OnAttached()
        {
            originalOpacity = Target.Opacity;
        }

        #region AlternateOpacity Property 

        /// <summary>
        /// Gets or sets the alternate opacity.
        /// </summary>
        /// <value>The alternate opacity.</value>
        [Description("Opacity when trigger in switched on.")]
        public double AlternateOpacity
        {
            get { return (double) GetValue(AlternateOpacityProperty); }
            set { SetValue(AlternateOpacityProperty, value); }
        }

        /// <summary>
        /// Gets or sets the alternate opacity.
        /// </summary>
        public static readonly DependencyProperty AlternateOpacityProperty =
            DependencyProperty.Register(
                "AlternateOpacity", typeof (double), typeof (ToggleOpacityAction),
                new PropertyMetadata(0.5d, null));
        
        #endregion

    }
}
