﻿using System.ComponentModel;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interactivity;

namespace Od.Behaviors.Commanding
{
    /// <summary>
    /// Associating a ExecuteCommand to any FrameworkElement allows to trigger an action.
    /// </summary>
    public class ExecuteCommandAction : TriggerAction<FrameworkElement>
    {

        [Category("ExecuteCommandAction Properties"),
        Description("The Command that will be executed by this TriggerAction")]
        public string TargetCommand
        {
            get { return (string)GetValue(TargetCommandProperty); }
            set { SetValue(TargetCommandProperty, value); }
        }

        public static readonly DependencyProperty TargetCommandProperty =
            DependencyProperty.Register("Target Command",
                                        typeof(string),
                                        typeof(ExecuteCommandAction),
                                        new PropertyMetadata
                                            (TargetCommandChanged));

        private static void TargetCommandChanged(DependencyObject d,
            DependencyPropertyChangedEventArgs e)
        {

        }

        protected override void Invoke(object parameter)
        {
            var path = TargetCommand;
            var dc = AssociatedObject.DataContext;
            ICommand targetCommand;
            try
            {
                targetCommand =
                   dc.GetType().GetProperty(path).GetValue(dc, null) as ICommand;
            }
            catch
            {
                targetCommand = null;
            }

            if (targetCommand != null && targetCommand.CanExecute(parameter))
            {
                targetCommand.Execute(parameter);
            }
        }
    }
}
