﻿using System.Windows;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media.Effects;

namespace Od.Behaviors.Effects
{
    public class BlurHoverBehavior : Behavior<UIElement>
    {
        readonly BlurEffect effect;

        public BlurHoverBehavior()
        {
            effect = new BlurEffect { Radius = BlurRadius };
        }

        #region BlurRadius (DependencyProperty)

        /// <summary>
        /// RadThe Blur Radius to use for the Blur Effect
        /// </summary>
        public double BlurRadius
        {
            get { return (double)GetValue(BlurRadiusProperty); }
            set { SetValue(BlurRadiusProperty, value); }
        }
        public static readonly DependencyProperty BlurRadiusProperty =
            DependencyProperty.Register("BlurRadius", typeof(double), typeof(BlurHoverBehavior),
            new PropertyMetadata((double)2, OnBlurRadiusChanged));

        private static void OnBlurRadiusChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((BlurHoverBehavior)d).OnBlurRadiusChanged(e);
        }

        protected virtual void OnBlurRadiusChanged(DependencyPropertyChangedEventArgs e)
        {
            effect.Radius = BlurRadius;
        }

        #endregion

        protected override void OnAttached()
        {
            base.OnAttached();

            AssociatedObject.MouseEnter += AssociatedObject_MouseEnter;
            AssociatedObject.MouseLeave += AssociatedObject_MouseLeave;
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();

            AssociatedObject.MouseEnter -= AssociatedObject_MouseEnter;
            AssociatedObject.MouseLeave -= AssociatedObject_MouseLeave;
        }

        #region Events
        void AssociatedObject_MouseLeave(object sender, MouseEventArgs e)
        {
            AssociatedObject.Effect = null;
        }

        void AssociatedObject_MouseEnter(object sender, MouseEventArgs e)
        {
            AssociatedObject.Effect = effect;
        }
        #endregion
    }
}
