﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Effects
{
    public enum HandledEvents
    {
        MouseOver, //MouseEnter - MouseLeave
        MouseClick, //MouseLeftButtonDown - MouseLeftButtonUp
        GotFocus //  //GotFocus - LostFocus
    }

    [Description("Loop3D - A behavior that enables a continuous 3D Loop on an item in response to an action.")]
    public class Loop3DAction : TargetedTriggerAction<FrameworkElement>
    {
        #region Behavior Properties
        private HandledEvents triggerEvent = HandledEvents.MouseOver;
        
        [Category("Projection Animation Properties")]
        public HandledEvents TriggerEvent
        {
            get { return triggerEvent; }
            set { triggerEvent = value; }
        }
       
        private TimeSpan rotateDurationX = TimeSpan.FromSeconds(1.8);
        private TimeSpan rotateDurationY = TimeSpan.FromSeconds(1.8);
        private TimeSpan rotateDurationZ = TimeSpan.FromSeconds(1.8);
       
        [Category("Projection Animation Properties")]
        public TimeSpan RotateDurationX
        {
            get { return rotateDurationX; }
            set { rotateDurationX = value; }
        }
        
        [Category("Projection Animation Properties")]
        public TimeSpan RotateDurationY
        {
            get { return rotateDurationY; }
            set { rotateDurationY = value; }
        }
        
        [Category("Projection Animation Properties")]
        public TimeSpan RotateDurationZ
        {
            get { return rotateDurationZ; }
            set { rotateDurationZ = value; }
        }
       
        [Category("Projection Animation Properties")]
        public IEasingFunction EasingX { get; set; }
        
        [Category("Projection Animation Properties")]
        public IEasingFunction EasingY { get; set; }
       
        [Category("Projection Animation Properties")]
        public IEasingFunction EasingZ { get; set; }

        private double rotateAngleX = 360;
        private double rotateAngleY = 360;

        [Category("Projection Animation Properties")]
        public double RotateAngleX
        {
            get { return rotateAngleX; }
            set { rotateAngleX = value; }
        }
        
        [Category("Projection Animation Properties")]
        public double RotateAngleY
        {
            get { return rotateAngleY; }
            set { rotateAngleY = value; }
        }

        [Category("Projection Animation Properties")]
        public double RotateAngleZ { get; set; }

        private RepeatBehavior rotateRepeat = new RepeatBehavior(1);
        
        [Category("Projection Animation Properties")]
        public RepeatBehavior RotateRepeatBehavior
        {
            get { return rotateRepeat; }
            set { rotateRepeat = value; }
        }

        [Category("Projection Animation Properties")]
        public bool RotateAutoReverse { get; set; }

        #endregion Behavior Properties

        #region private variables
        private TimeSpan tsAnimationTimeX;
        private TimeSpan tsAnimationTimeY;
        private TimeSpan tsAnimationTimeZ;
        private FrameworkElement feSourceObject;
        private FrameworkElement feTargetObject;
        private PlaneProjection projectionTargetObject;
        #endregion private variables

        protected override void Invoke(object parameter)
        {
        }


        protected override void OnAttached()
        {
            base.OnAttached();
            feSourceObject = (FrameworkElement)AssociatedObject;
            tsAnimationTimeX = rotateDurationX;
            tsAnimationTimeY = rotateDurationY;
            tsAnimationTimeZ = rotateDurationZ;
            feSourceObject.Loaded += feSourceObject_Loaded;
        }

       
        void feSourceObject_Loaded(object sender, RoutedEventArgs e)
        {
            feSourceObject.Loaded -= feSourceObject_Loaded;

            projectionTargetObject = new PlaneProjection();
            feTargetObject = (FrameworkElement)TargetObject;

            if (feTargetObject.Projection == null)
            {
                feTargetObject.RenderTransformOrigin = new Point(0.5, 0.5);
                var pj = new PlaneProjection();
                feTargetObject.Projection = pj;
            }

            projectionTargetObject.RotationX = ((PlaneProjection)feTargetObject.Projection).RotationX;
            projectionTargetObject.RotationY = ((PlaneProjection)feTargetObject.Projection).RotationY;
            projectionTargetObject.RotationZ = ((PlaneProjection)feTargetObject.Projection).RotationZ;

            AttachEventsToTargetObject();
        }

        private void AttachEventsToTargetObject()
        {
            switch (triggerEvent)
            {
                case HandledEvents.MouseOver:
                    feSourceObject.MouseEnter += feSourceObject_MouseEnter;
                    feSourceObject.MouseLeave += feSourceObject_MouseLeave;
                    break;
                case HandledEvents.MouseClick:
                    feSourceObject.MouseLeftButtonDown += feSourceObject_MouseLeftButtonDown;
                    feSourceObject.MouseLeftButtonUp += feSourceObject_MouseLeftButtonUp;
                    break;
                case HandledEvents.GotFocus:
                    feSourceObject.GotFocus += feSourceObject_GotFocus;
                    feSourceObject.LostFocus += feSourceObject_LostFocus;
                    break;
                default:
                    break;
            }

        }

        protected override void OnDetaching()
        {
            base.OnDetaching();
            if (feSourceObject != null)
            {
                switch (triggerEvent)
                {
                    case HandledEvents.MouseOver:
                        feSourceObject.MouseEnter -= feSourceObject_MouseEnter;
                        feSourceObject.MouseLeave -= feSourceObject_MouseLeave;
                        break;
                    case HandledEvents.MouseClick:
                        feSourceObject.MouseLeftButtonDown -= feSourceObject_MouseLeftButtonDown;
                        feSourceObject.MouseLeftButtonUp -= feSourceObject_MouseLeftButtonUp;
                        break;
                    case HandledEvents.GotFocus:
                        feSourceObject.GotFocus -= feSourceObject_GotFocus;
                        feSourceObject.LostFocus -= feSourceObject_LostFocus;
                        break;
                    default:
                        break;
                }
            }
        }


        void feSourceObject_MouseLeave(object sender, MouseEventArgs e)
        {
            DeactivateAnimation();
        }
        void feSourceObject_MouseEnter(object sender, MouseEventArgs e)
        {
            ActivateAnimation();
        }
        void feSourceObject_LostFocus(object sender, RoutedEventArgs e)
        {
            DeactivateAnimation();
        }

        void feSourceObject_GotFocus(object sender, RoutedEventArgs e)
        {
            ActivateAnimation();
        }

        void feSourceObject_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            DeactivateAnimation();
        }

        void feSourceObject_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            ActivateAnimation();
        }

        bool bAnimationActivated;
        private void ActivateAnimation()
        {
            if (bAnimationActivated) return;
            AnimateToAngleXYZ(RotateAngleX, RotateAngleY, RotateAngleZ, RotateRepeatBehavior, RotateAutoReverse);
            bAnimationActivated = true;
        }
        private void DeactivateAnimation()
        {
            if (!bAnimationActivated) return;
            AnimateToAngleXYZ(0, 0, 0, new RepeatBehavior(1), false);
            bAnimationActivated = false;
        }


        private void AnimateToAngleXYZ(Double x, Double y, Double z, RepeatBehavior repeatBehavior, bool bAutoReverse)
        {
           if ((x == y) && (y == z) && (x == 0))
            {
                playAnimation(feTargetObject, "(UIElement.Projection).(PlaneProjection.RotationX)", tsAnimationTimeX, null, x,  EasingX, repeatBehavior, bAutoReverse);
                playAnimation(feTargetObject, "(UIElement.Projection).(PlaneProjection.RotationY)", tsAnimationTimeY, null, y,  EasingY, repeatBehavior, bAutoReverse);
                playAnimation(feTargetObject, "(UIElement.Projection).(PlaneProjection.RotationZ)", tsAnimationTimeZ, null, z,  EasingZ, repeatBehavior, bAutoReverse);
            }
            else
            {
                playAnimation(feTargetObject, "(UIElement.Projection).(PlaneProjection.RotationX)", tsAnimationTimeX, 0, x,  EasingX, repeatBehavior, bAutoReverse);
                playAnimation(feTargetObject, "(UIElement.Projection).(PlaneProjection.RotationY)", tsAnimationTimeY, 0, y,  EasingY, repeatBehavior, bAutoReverse);
                playAnimation(feTargetObject, "(UIElement.Projection).(PlaneProjection.RotationZ)", tsAnimationTimeZ, 0, z,  EasingZ, repeatBehavior, bAutoReverse);
            }
        }

        private static void playAnimation(FrameworkElement element, string property, TimeSpan time, double? valueFrom, double valueTo, 
                                         IEasingFunction easingFunction, RepeatBehavior repeatBehavior, bool bAutoReverse)
        {
            var sb = new Storyboard();
            
            // create a DoubleAnimation for the storyboard, set the properties and add it to the storyboard
            var animation = new DoubleAnimation
                                {
                                    Duration = time,
                                    From = valueFrom,
                                    To = valueTo,
                                    EasingFunction = easingFunction,
                                    AutoReverse = bAutoReverse,
                                    RepeatBehavior = repeatBehavior
                                };
            Storyboard.SetTargetProperty(animation, new PropertyPath(property));
            Storyboard.SetTarget(animation, element);
            sb.Children.Add(animation);
            // finally start the animation
            sb.Begin();
        }

    }
}
