﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Effects
{
    [Description("MouseOver3D - A behavior that applies the MouseOver Design Interaction Pattern to a 3D environment.")]
    public class MouseOver3DAction : TargetedTriggerAction<FrameworkElement>
    {
        private TimeSpan hoverUpDuration = TimeSpan.FromSeconds(0.5);
        [Category("Mouse Over 3D - Going Up")]
        public TimeSpan HoverUp_duration
        {
            get { return hoverUpDuration; }
            set { hoverUpDuration = value; }
        }
        private TimeSpan hoverDownDuration = TimeSpan.FromSeconds(0.9);
        [Category("Mouse Over 3D - Going Down")]
        public TimeSpan HoverDown_Duration
        {
            get { return hoverDownDuration; }
            set { hoverDownDuration = value; }
        }
        [Category("Mouse Over 3D - Going Up")]
        public IEasingFunction HoverUp_Easing { get; set; }
        [Category("Mouse Over 3D - Going Down")]
        public IEasingFunction HoverDown_Easing { get; set; }
        private double hoverOffset = 30;
        [Category("Mouse Over 3D - General")]
        public double HoverOffset
        {
            get { return hoverOffset; }
            set { hoverOffset = value; }
        }
        private FrameworkElement feAssociatedObject;
        private FrameworkElement feSourceObject;
        private FrameworkElement feTargetObject;


        protected override void Invoke(object parameter)
        {
        }

        protected override void OnAttached()
        {
            base.OnAttached();
            feAssociatedObject = (FrameworkElement)AssociatedObject;
            feSourceObject = (FrameworkElement)AssociatedObject;
            feSourceObject.Loaded += feSourceObject_Loaded;
        }
        void feSourceObject_Loaded(object sender, RoutedEventArgs e)
        {
            feSourceObject.Loaded -= feSourceObject_Loaded;
            feTargetObject = (FrameworkElement)TargetObject ?? feAssociatedObject;
            if (feTargetObject.Projection == null)
            {
                feTargetObject.RenderTransformOrigin = new Point(0.5, 0.5);
                feTargetObject.Projection = new PlaneProjection();
            }
            feSourceObject.MouseEnter += feSourceObject_MouseEnter;
            feSourceObject.MouseLeave += feSourceObject_MouseLeave;
        }

        protected override void OnDetaching()
        {
            feSourceObject.Loaded -= feSourceObject_Loaded; 
            base.OnDetaching();
        }
        
        void feSourceObject_MouseLeave(object sender, MouseEventArgs e)
        {
            DeactivateAnimation();
        }
        
        void feSourceObject_MouseEnter(object sender, MouseEventArgs e)
        {
            ActivateAnimation();
        }

        bool bAnimationActivated;
        private void ActivateAnimation()
        {
            if (bAnimationActivated) return;
            AnimateHoverZ(HoverOffset, true);
            bAnimationActivated = true;
        }
        private void DeactivateAnimation()
        {
            if (!bAnimationActivated) return;
            AnimateHoverZ(0, false);
            bAnimationActivated = false;
        }

        private void AnimateHoverZ(Double z, bool hoverUp)
        {
            if (hoverUp)
                playAnimation(feTargetObject, "(UIElement.Projection).(PlaneProjection.LocalOffsetZ)", HoverUp_duration, z,  HoverUp_Easing);
            else
                playAnimation(feTargetObject, "(UIElement.Projection).(PlaneProjection.LocalOffsetZ)", HoverDown_Duration, z, HoverDown_Easing);
        }
        public static void playAnimation(FrameworkElement element, string property, TimeSpan time, double value, IEasingFunction easingFunction)
        {
            var sb = new Storyboard();
            sb.Children.Clear();
            var animation = new DoubleAnimation {Duration = time, To = value, EasingFunction = easingFunction};
            Storyboard.SetTargetProperty(animation, new PropertyPath(property));
            Storyboard.SetTarget(animation, element);
            sb.Children.Add(animation);
            sb.Begin();
        }

    }
}
