﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;

namespace Od.Behaviors.Function
{
    public class DragBehavior : Behavior<UIElement>
    {
        private bool isDragging;
        private UIElement attachedElement;
        private UserControl parent;

        Point lastPosition;

        protected override void OnAttached()
        {
            attachedElement = AssociatedObject;
            parent = Application.Current.RootVisual as UserControl;

            attachedElement.MouseLeftButtonDown += MouseIsDown;
            attachedElement.MouseLeftButtonUp += MouseIsUp;
            attachedElement.MouseMove += MouseIsMoving;
        }

        void MouseIsMoving(object sender, MouseEventArgs e)
        {
            if (!isDragging) return;
            var currentPosition = e.GetPosition(parent);

            var dX = currentPosition.X - lastPosition.X;
            var dY = currentPosition.Y - lastPosition.Y;

            lastPosition = currentPosition;

            var oldTransform = attachedElement.RenderTransform;
            var rt = new TransformGroup();
            var newPos = new TranslateTransform {X = dX, Y = dY};

            if (oldTransform != null)
            {
                rt.Children.Add(oldTransform);
            }
            rt.Children.Add(newPos);

            var mt = new MatrixTransform {Matrix = rt.Value};

            attachedElement.RenderTransform = mt;
        }

        void MouseIsUp(object sender, MouseButtonEventArgs e)
        {
            isDragging = false;

            attachedElement.ReleaseMouseCapture();
        }

        void MouseIsDown(object sender, MouseButtonEventArgs e)
        {
            isDragging = true;
            lastPosition = e.GetPosition(parent);
            attachedElement.CaptureMouse();
        }
    }
}
