﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Interactivity;

namespace Od.Behaviors.Function
{
    public class SetFocusAction : TargetedTriggerAction<Control>
    {
        protected override void Invoke(object parameter)
        {
            if (!IsApplied)
                if (Target != null) Target.Focus();

            if (IsOneShot)
                IsApplied = true;
        }

        #region IsOneShot
        public static readonly DependencyProperty IsOneShotProperty = DependencyProperty.Register(
                   "IsOneShot", typeof(bool), typeof(SetFocusAction), new PropertyMetadata(false,onPropertyChangedCallback));

        private static void onPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((SetFocusAction) d).IsApplied = false; // reset internal flag
        }

        /// <summary>
        /// Gets or sets a value indicating whether the focus is set only one time.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is one shot; otherwise, <c>false</c>.
        /// </value>
        public bool IsOneShot
        {
            get { return (bool)GetValue(IsOneShotProperty); }
            set { SetValue(IsOneShotProperty, value); }
        }
        #endregion


        /// <summary>
        /// Gets or sets a value indicating whether focus has been applied.
        /// (always false when IsOneShot is set to false).
        /// This property is used to reset the behavior (can be usefull).
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is applied; otherwise, <c>false</c>.
        /// </value>
        public bool IsApplied { get; set; }
    }
}
