﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Utilities
{
    public class AnimationMaker
    {
        // this method will create a new storyboard and start the animation
        public static void PlayAnimation(UIElement element, string elementName, string property, TimeSpan time, double value)
        {
            // create a new storyboard for the animation and set the targetname and property
            var sb = new Storyboard();
            sb.SetValue(Storyboard.TargetNameProperty, elementName);
            sb.SetValue(Storyboard.TargetPropertyProperty, new PropertyPath(property));

            // create a DoubleAnimation for the storyboard, set the properties and add it to the storyboard
            var animation = new DoubleAnimation { Duration = time, To = value };
            sb.Children.Add(animation);

            // create a unique name
            var name = "PaSb" + Guid.NewGuid().ToString("N");

            DependencyObject p = element;
            do
            {
                p = VisualTreeHelper.GetParent(p);
            } while (p != null && (!(p is Control)));

            if (p != null && (p is Control))
            {
                ((Control)p).Resources.Remove(name);
                ((Control)p).Resources.Add(name, sb);
            }
            else
            {
                p = null;
                // add the animation to the resources -> required to play the animation
                ((UserControl)Application.Current.RootVisual).Resources.Remove(name);
                ((UserControl)Application.Current.RootVisual).Resources.Add(name, sb);
            }

            sb.Completed += (o, e) =>
                                {
                                    if (p==null)
                                    ((UserControl) Application.Current.RootVisual).Resources.Remove(name);
                                    else ((Control)p).Resources.Remove(name);
                                };

            // finally start the animation
            sb.Begin();
        }
    }
}
