﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Utilities
{
    public static class StoryBoardHelper
    {
        public static void ScaleAnimation(UIElement controlToAnimate, double factor, double duration)
        {
            var story = new Storyboard();

            //stretch horizontally
            var scaleXAnimation = new DoubleAnimationUsingKeyFrames {BeginTime = TimeSpan.FromMilliseconds(0)};
            scaleXAnimation.KeyFrames.Add(CreateFrame(factor, duration));
            Storyboard.SetTarget(scaleXAnimation, controlToAnimate);
            Storyboard.SetTargetProperty(scaleXAnimation, new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[0].(ScaleTransform.ScaleX)"));
            story.Children.Add(scaleXAnimation);

            //stretch vertically
            var scaleYAnimation = new DoubleAnimationUsingKeyFrames {BeginTime = TimeSpan.FromMilliseconds(0)};
            scaleYAnimation.KeyFrames.Add(CreateFrame(factor, duration)); 
            Storyboard.SetTarget(scaleYAnimation, controlToAnimate);
            Storyboard.SetTargetProperty(scaleYAnimation, new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[0].(ScaleTransform.ScaleY)"));
            story.Children.Add(scaleYAnimation);

            if (!(controlToAnimate.RenderTransform is TransformGroup))
            {
                var group = new TransformGroup();
                var transform = new ScaleTransform {ScaleX = 1, ScaleY = 1};
                group.Children.Add(transform);
                controlToAnimate.RenderTransformOrigin = new Point(0.5, 0.5);
                controlToAnimate.RenderTransform = group;
            }
            story.Begin();


        }

        private static SplineDoubleKeyFrame CreateFrame(double value, double duration)
        {
            var frame = new SplineDoubleKeyFrame {Value = value, KeyTime = TimeSpan.FromMilliseconds(duration)};
            return frame;
        }
    }
}
