﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Utilities
{
    public class TransitionHelper
    {
        public static TransformGroup GetTransformGroup(double x, double y, double scaleX, double scaleY)
        {
            var tt = new TranslateTransform { X = x, Y = y };

            var rt = new RotateTransform { Angle = 0.00D };

            var st = new ScaleTransform { ScaleX = scaleX, ScaleY = scaleY };

            var tg = new TransformGroup();
            tg.Children.Add(st);
            tg.Children.Add(rt);
            tg.Children.Add(tt);

            return tg;
        }

        public static Storyboard BuildTranslateTransformTransition(UIElement element, Point newLocation, TimeSpan period, bool autoReverse)
        {
            var duration = new Duration(period);

            var expo = new ExponentialEase
                           {
                               EasingMode = EasingMode.EaseInOut,
                           };

            // Animate X
            var translateAnimationX = new DoubleAnimation
                                          {
                                              To = newLocation.X,
                                              Duration = duration,
                                              EasingFunction = expo
                                          };

            Storyboard.SetTarget(translateAnimationX, element);
            Storyboard.SetTargetProperty(translateAnimationX,
                                         new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[2].(TranslateTransform.X)"));

            // Animate Y
            var translateAnimationY = new DoubleAnimation { To = newLocation.Y, Duration = duration };
            //translateAnimationY.EasingFunction = ease;

            Storyboard.SetTarget(translateAnimationY, element);
            Storyboard.SetTargetProperty(translateAnimationY,
                new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[2].(TranslateTransform.Y)"));

            var sb = new Storyboard { AutoReverse = autoReverse, Duration = duration };
            sb.Children.Add(translateAnimationX);
            sb.Children.Add(translateAnimationY);

            return sb;
        }

        public static Storyboard BuildVisibilityAnimation(UIElement element, Double newOpacity, Double newHeight,
                                                          TimeSpan opacityPeriod, TimeSpan heightPeriod, bool firstHeight)
        {
            var expoOut = new ExponentialEase
                              {
                                  EasingMode = EasingMode.EaseOut,
                              };
            var expoIn = new ExponentialEase
                             {
                                 EasingMode = EasingMode.EaseIn,
                             };

            // Animate Opacity
            var opacityAnimation = new DoubleAnimation
                                       {
                                           To = newOpacity,
                                           Duration = new Duration(opacityPeriod),
                                           EasingFunction = expoOut
                                       };
            if (firstHeight) opacityAnimation.BeginTime = heightPeriod;

            Storyboard.SetTarget(opacityAnimation, element);
            Storyboard.SetTargetProperty(opacityAnimation,
                                         new PropertyPath(UIElement.OpacityProperty));

            // Animate Height
            var heightAnimation = new DoubleAnimation
                                      {
                                          To = newHeight,
                                          Duration = new Duration(heightPeriod),
                                          EasingFunction = expoIn
                                      };
            if (!firstHeight) heightAnimation.BeginTime = opacityPeriod;

            Storyboard.SetTarget(heightAnimation, element);
            Storyboard.SetTargetProperty(heightAnimation,
                                         new PropertyPath(FrameworkElement.HeightProperty));


            var sb = new Storyboard { AutoReverse = false, Duration = new Duration(opacityPeriod + heightPeriod) };
            sb.Children.Add(opacityAnimation);
            sb.Children.Add(heightAnimation);

            return sb;
        }
    }
}